import javax.swing.*;
import javax.swing.plaf.basic.BasicScrollBarUI;
import javax.swing.plaf.metal.MetalScrollBarUI;
import java.awt.*;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.*;
import java.nio.charset.StandardCharsets;
import java.util.LinkedList;
import java.util.List;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.stream.Collectors;

/**
 * CREATED BY liusu
 */
public class RunToday{

    public static void main(String[] args) throws Exception {

        JFrame jf = new JFrame("kawachi 前线新闻");
        jf.setSize(700,600);
        jf.setIconImage(new ImageIcon("icon.jpg").getImage());

        JToolBar toolBar = new JToolBar();
        JButton fresh = new JButton("刷新");
        JButton editBanned = new JButton("过滤");
        JButton editOrder = new JButton("排序");
        JButton author = new JButton("关于");
        toolBar.add(fresh);
        toolBar.add(editBanned);
        toolBar.add(editOrder);
        toolBar.add(author);
        toolBar.setFloatable(false);

        JPanel panel = new JPanel();
        panel.setLayout(new GridLayout(0, 1));

        drawPanel(getList(), panel);

        JScrollPane scrollPane = new JScrollPane(panel);
        scrollPane.getVerticalScrollBar().setUnitIncrement(10);

        scrollPane.getVerticalScrollBar().setUI(new BasicScrollBarUI() {
            @Override
            protected void configureScrollBarColors() {
                this.thumbColor = Color.PINK;
            }
        });

        jf.add(toolBar, BorderLayout.NORTH);
        jf.add(scrollPane, BorderLayout.CENTER);

        jf.setLocationRelativeTo(null);
        jf.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        jf.setVisible(true);

        fresh.addActionListener( e ->
            {
                panel.removeAll();
                panel.updateUI();
                try {
                    drawPanel(getList(), panel);
                } catch (URISyntaxException | IOException uriSyntaxException) {
                    uriSyntaxException.printStackTrace();
                }

                SwingUtilities.invokeLater(() -> scrollPane.getVerticalScrollBar().setValue(0));
            }
        );
        editBanned.addActionListener(e -> {
            new EditFrame("编辑过滤器", "banned.txt").setVisible(true);
        });
        editOrder.addActionListener(e -> {
            new EditFrame("编辑排序器", "order.txt").setVisible(true);
        });
        author.addActionListener(e -> {
            JOptionPane.showMessageDialog(null, "<html>作者: 时光潜流<br/></html>名称: kawachi 前线新闻");
        });
    }

    private static List<OneNews> getList() throws URISyntaxException, IOException {
        URL url = new URI("https://top.baidu.com/board?tab=realtime").toURL();
        URLConnection urlConnection = url.openConnection();
        HttpURLConnection conn = (HttpURLConnection) urlConnection;

        StringBuilder sb = new StringBuilder();
        try (BufferedReader br = new BufferedReader(new InputStreamReader(conn.getInputStream(), StandardCharsets.UTF_8))){
            String tmp;
            while ((tmp = br.readLine()) != null) {
                sb.append(tmp);
            }
        } catch (Exception e){
            e.printStackTrace();
        }


        List<OneNews> list = new LinkedList<>();

        Pattern patternTitle = Pattern.compile("<div class=\"c-single-text-ellipsis\">(.+?) </div>");
        Pattern patternUrl = Pattern.compile("<a[^>]*href=\"([^\"]*)\".class=\"title_dIF3B.*?>");
        Pattern patternContent = Pattern.compile("<!--s-frag-->[^\"].*?<div class=\".*? \">[^\"](.*?)<");
        Matcher m1 = patternTitle.matcher(sb);
        Matcher m2 = patternUrl.matcher(sb);
        Matcher m3 = patternContent.matcher(sb);
        while (m1.find() && m2.find() && m3.find()){
            OneNews tmp = new OneNews();

            tmp.setTitle(m1.group(1).trim());
            tmp.setUrl(m2.group(1).trim());
            tmp.setContent(m3.group(1).trim());
            tmp.setValue(0);

            // 获取权重
            for (String valueInit : NewsComparator.valueInits) {
                if (tmp.getTitle().contains(valueInit)) {
                    tmp.setValue(tmp.getValue() + 1);
                }
            }

            list.add(tmp);
        }

        return list.stream().sorted(new NewsComparator()).filter(new NewsPredicate()).collect(Collectors.toList());
    }

    private static void drawPanel(List<OneNews> list, JPanel panel) {
        Font font = new Font("宋体", Font.BOLD, 16);
        AtomicInteger i = new AtomicInteger();
        list.forEach(item -> {
            i.getAndIncrement();
            JButton tmp = new JButton("<html><u>  " + (i.get() < 10 ? "0" + i : i) + "." + item.getTitle() + "</u></html>");
            tmp.setBackground(Color.WHITE);
            tmp.setFont(font);
            tmp.setForeground(Color.BLUE);
            tmp.setHorizontalAlignment(JButton.LEFT);
            tmp.setBorder(null);
            tmp.addActionListener(e -> {
                Desktop desktop = Desktop.getDesktop();
                try {
                    desktop.browse(new URI(item.getUrl()));
                } catch (IOException | URISyntaxException ioException) {
                    ioException.printStackTrace();
                }
            });

            JTextArea jta = new JTextArea("    " + (item.getContent().equals("") ? "-" : item.getContent()));
            jta.setLineWrap(true);
            jta.setEditable(false);
            jta.setFont(new Font("宋体", Font.PLAIN, 16));

            panel.add(tmp);
            panel.add(jta);
        });
    }
}

